@@ -126,12 +126,16 @@ report_newcon(port_info_t *port, net_info_t *netcon)
     if (!net_raddr_str(netcon->net, netcon->remaddr, sizeof(netcon->remaddr)))
 	strcpy(netcon->remaddr, "*unknown*");
     cntlr_report_conchange("new connection", port->name, netcon->remaddr);
+    if (port->led_conn)
+	led_enable(port->led_conn, 1);
 }
 
 void
 report_disconnect(port_info_t *port, net_info_t *netcon)
 {
     cntlr_report_conchange("disconnect", port->name, netcon->remaddr);
+    if (port->led_conn)
+	led_enable(port->led_conn, 0);
 }
 
 static void
@@ -588,8 +592,6 @@ handle_net_fd_read(net_info_t *netcon, int readerr,
 	port->net_to_dev_state = PORT_WAITING_OUTPUT_CLEAR;
     }
 
-    reset_timer(netcon);
-
     rv = buflen;
 
  out_unlock:
@@ -646,7 +648,7 @@ net_fd_write(port_info_t *port, net_info_t *netcon,
     return 1;
 }
 
-static void
+void
 finish_dev_to_net_write(port_info_t *port)
 {
     if (any_net_data_to_write(port))
@@ -699,8 +701,6 @@ handle_net_fd_write_ready(net_info_t *netcon)
     if (rv != 0)
 	gensio_set_write_callback_enable(netcon->net, false);
 
-    if (rv >= 0)
-	reset_timer(netcon);
     so->unlock(port->lock);
 }
 
