@@ -233,6 +233,12 @@ free_leds(void)
     unlock_leds();
 }
 
+int
+led_enable(struct led_s *led, int value)
+{
+    return led->driver->enable(led->drv_data, value);
+}
+
 int
 led_flash(struct led_s *led)
 {
