@@ -29,6 +29,7 @@
 #include <assert.h>
 #include "ser2net.h"
 #include "port.h"
+#include "dataxfer.h"
 #include "gbuf.h"
 #include <gensio/argvutils.h>
 #include <gensio/gensio_err.h>
@@ -109,7 +110,8 @@ net_raddr(struct gensio *io, struct sockaddr_storage *addr, gensiods *socklen)
 void
 reset_timer(net_info_t *netcon)
 {
-    if (netcon->connect_back && netcon->port->connback_timeout_set)
+    if (netcon->connect_back && netcon->port->connback_timeout_set
+		&& !netcon->net)
 	netcon->timeout_left = netcon->port->connback_timeout;
     else
 	netcon->timeout_left = netcon->port->timeout;
@@ -550,6 +552,12 @@ netcon_finish_shutdown(net_info_t *netcon)
 	}
     } else {
 	check_port_new_net(port, netcon);
+	if (netcon->port->dev_to_net_state == PORT_WAITING_OUTPUT_CLEAR)
+	    /*
+	     * If we are blocked on this writer, make sure that is
+	     * cleaned up.
+	     */
+	    finish_dev_to_net_write(netcon->port);
     }
 }
 
@@ -787,6 +795,47 @@ port_startup(port_info_t *new_port, struct absout *eout, bool retry)
     return err;
 }
 
+/*
+ * Return if any data has been sent on the port.  It also returns (in
+ * port_stuck) if data is pending on the port and the other end is not
+ * receiving them.
+ */
+static bool
+port_data_sent(net_info_t *netcon, bool *port_stuck)
+{
+    int rv;
+    char countstr[20];
+    gensiods count, nc;
+
+    *port_stuck = false;
+    if (!netcon->port->timeout_on_os_queue)
+	goto out;
+
+    /*
+     * This fetches the total number of bytes outstanding in the OS
+     * queues.
+     */
+    count = sizeof(countstr);
+    rv = gensio_control(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+			GENSIO_CONTROL_GET, GENSIO_CONTROL_DRAIN_COUNT,
+			countstr, &count);
+    if (rv)
+	goto out;
+
+    /*
+     * The number of bytes we sent in the last period plus the last
+     * queue length is the number of bytes that would be pending if
+     * nothing was sent out of the OS queue.
+     */
+    count = strtoul(countstr, NULL, 0);
+    nc = netcon->bytes_sent - netcon->last_bytes_sent
+	+ netcon->last_send_queue_len;
+    netcon->last_send_queue_len = count;
+    *port_stuck = count > 0 && nc == count;
+ out:
+    return netcon->bytes_sent != netcon->last_bytes_sent;
+}
+
 static void
 port_timeout(struct gensio_timer *timer, void *data)
 {
@@ -846,12 +895,33 @@ port_timeout(struct gensio_timer *timer, void *data)
      * timeout for connect backs, we need to check that separately.
      */
     if (port_in_use(port)) {
+	bool port_stuck, any_port_stuck = false, data_sent;
+
+	for_each_connection(port, netcon) {
+	    port_data_sent(netcon, &port_stuck);
+	    if (port_stuck)
+		any_port_stuck = true;
+	}
 	for_each_connection(port, netcon) {
 	    if (!netcon->net || !netcon->timeout_running)
 		continue;
-	    netcon->timeout_left--;
-	    if (netcon->timeout_left < 0)
-		shutdown_one_netcon(netcon, "timeout");
+	    if (netcon->bytes_received != netcon->last_bytes_received) {
+		reset_timer(netcon);
+	    } else {
+		data_sent = port_data_sent(netcon, &port_stuck);
+		/* If any port is stuck, we only consider stuck ports. */
+		if (any_port_stuck)
+		    data_sent = !port_stuck;
+		if (data_sent) {
+		    reset_timer(netcon);
+		} else {
+		    netcon->timeout_left--;
+		    if (netcon->timeout_left < 0)
+			shutdown_one_netcon(netcon, "timeout");
+		}
+	    }
+	    netcon->last_bytes_received = netcon->bytes_received;
+	    netcon->last_bytes_sent = netcon->bytes_sent;
 	}
     }
 
