@@ -333,6 +333,8 @@ finish_free_port(port_info_t *port)
 	free_led(port->led_rx);
     if (port->led_tx)
 	free_led(port->led_tx);
+    if (port->led_conn)
+	free_led(port->led_conn);
 #ifdef DO_MDNS
     mdns_shutdown(&port->mdns_info);
 #endif /* DO_MDNS */
@@ -435,6 +437,8 @@ myconfig(port_info_t *port, struct absout *eout, const char *pos)
 
     if (gensio_check_keybool(pos, "kickolduser",
 			     &port->kickolduser_mode) > 0) {
+    } else if (gensio_check_keybool(pos, "timeout-on-os-queue",
+				    &port->timeout_on_os_queue) > 0) {
     } else if (gensio_check_keybool(pos, "trace-hexdump",
 				    &port->trace_read.hexdump) > 0) {
 	port->trace_write.hexdump = port->trace_read.hexdump;
@@ -482,6 +486,13 @@ myconfig(port_info_t *port, struct absout *eout, const char *pos)
 	    eout->out(eout, "Could not find led-tx LED: %s", val);
 	    return -1;
 	}
+    } else if (gensio_check_keyvalue(pos, "led-conn", &val) > 0) {
+	/* LED for UART Connection */
+	port->led_conn = find_led(val);
+	if (!port->led_conn) {
+	    eout->out(eout, "Could not find led-conn LED: %s", val);
+	    return -1;
+	}
     } else if (gensio_check_keybool(pos, "telnet-brk-on-sync",
 				    &port->telnet_brk_on_sync) > 0) {
     } else if (gensio_check_keybool(pos, "chardelay",
@@ -656,6 +667,7 @@ init_port_data(port_info_t *port, struct absout *eout)
 
     port->telnet_brk_on_sync = find_default_bool("telnet-brk-on-sync");
     port->kickolduser_mode = find_default_bool("kickolduser");
+    port->timeout_on_os_queue = find_default_bool("timeout-on-os-queue");
     port->enable_chardelay = find_default_int("chardelay");
     port->chardelay_scale = find_default_int("chardelay-scale");
     port->chardelay_min = find_default_int("chardelay-min");
@@ -686,6 +698,7 @@ init_port_data(port_info_t *port, struct absout *eout)
 
     port->led_tx = NULL;
     port->led_rx = NULL;
+    port->led_conn = NULL;
 
 #ifdef DO_MDNS
     mdns_info_getdefaults(&port->mdns_info, port->name, eout);
