@@ -28,12 +28,6 @@
 #include <stdarg.h>
 #include <stdbool.h>
 #include <yaml.h>
-#include <syslog.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <errno.h>
 #if defined(HAVE_WORDEXP) && defined(HAVE_WORDEXP_H)
 #define DO_WORDEXP
 #endif
@@ -46,8 +40,26 @@
 #include "dataxfer.h"
 #include "readconfig.h"
 #include "led.h"
+#include "fileio.h"
 
-//#define DEBUG 1
+//#define DEBUG 1//
+
+#ifdef WIN32
+char *strndup(const char *s, size_t n)
+{
+    int len = strlen(s);
+    char *r;
+
+    if (len > n)
+	len = n;
+    r = malloc(len + 1);
+    if (!r)
+	return NULL;
+    memcpy(r, s, len);
+    r[len] = '\0';
+    return r;
+}
+#endif
 
 /*
  * States for the state machine reading the configuration file.
@@ -61,9 +73,39 @@ enum ystate {
      */
     MAIN_LEVEL,
 
-    IN_DEFINE,
     IN_INCLUDE,
 
+    IN_DEFINES,
+    IN_DEFINES_MAP,
+    IN_DEFINES_VAL,
+
+    IN_DELDEFAULTS,
+    IN_DELDEFAULTS_MAP,
+    IN_DELDEFAULTS_SEQ,
+    IN_DELDEFAULTS_VAL,
+
+    IN_DEFAULTS,
+    IN_DEFAULTS_MAP,
+    IN_DEFAULTS_INST,
+
+    IN_ADMINS,
+    IN_ADMINS_MAP,
+    IN_ADMINS_INST,
+
+    IN_LEDS,
+    IN_LEDS_MAP,
+    IN_LEDS_INST,
+
+    IN_CONNECTIONS,
+    IN_CONNECTIONS_MAP,
+    IN_CONNECTIONS_INST,
+
+    IN_ROTATORS,
+    IN_ROTATORS_MAP,
+    IN_ROTATORS_INST,
+
+    IN_DEFINE,
+
     /*
      * We have read the type of the main mapping (connection, led, etc)
      * now waiting for a mapping start.
@@ -141,9 +183,9 @@ struct keyval_info {
 struct map_info {
     char *name;
     struct scalar_next_state *states;
-    enum ystate next_state;
     int map_type;
     bool needs_anchor;
+    bool wants_anchor;
 };
 
 enum which_info {
@@ -162,6 +204,7 @@ struct scalar_next_state {
 	struct keyval_info *keyval_info;
 	struct map_info *map_info;
     };
+    int version;
 };
 
 /*
@@ -179,7 +222,7 @@ struct yaml_read_file {
     yaml_parser_t parser;
     yaml_event_t e;
     char *filename;
-    FILE *f;
+    ftype *ft;
 
     struct yaml_read_file *prev_f;
 };
@@ -199,6 +242,7 @@ struct yaml_read_handler_data {
 
 struct yconf {
     enum ystate state;
+    int version;
 
     struct map_info *map_info;
 
@@ -214,9 +258,9 @@ struct yconf {
     unsigned int timeout;
     bool enable;
 
-    char **connections;
-    unsigned int curr_connection;
-    unsigned int connections_len;
+    char **seq;
+    unsigned int curr_seq;
+    unsigned int seq_len;
 
     struct option_info *option_info;
     char **options;
@@ -260,6 +304,16 @@ yaml_errout(struct yconf *y, const char *str, ...)
     return rv;
 }
 
+static int
+sub_verrout(struct absout *e, const char *str, va_list ap)
+{
+    struct yconf *y = e->data;
+    int rv;
+
+    rv = yaml_verrout_d_f(y->d, y->d->f, str, ap);
+    return rv;
+}
+
 static int
 sub_errout(struct absout *e, const char *str, ...)
 {
@@ -318,15 +372,15 @@ yconf_cleanup_main(struct yconf *y)
 	}
     }
     y->curr_option = 0;
-    if (y->connections) {
+    if (y->seq) {
 	unsigned int i;
 
-	for (i = 0; i < y->curr_connection && y->connections[i]; i++) {
-	    free(y->connections[i]);
-	    y->connections[i] = NULL;
+	for (i = 0; i < y->curr_seq && y->seq[i]; i++) {
+	    free(y->seq[i]);
+	    y->seq[i] = NULL;
 	}
     }
-    y->curr_connection = 0;
+    y->curr_seq = 0;
 }
 
 static struct alias *
@@ -393,8 +447,8 @@ cleanup_yaml_read_file(struct yaml_read_handler_data *d)
     d->f = d->f->prev_f;
     yaml_parser_delete(&old_f->parser);
     if (old_f->closeme) {
-	if (old_f->f)
-	    fclose(old_f->f);
+	if (old_f->ft)
+	    f_close(old_f->ft);
 #ifdef DO_WORDEXP
 	if (old_f->files_set)
 	    wordfree(&old_f->files);
@@ -427,13 +481,15 @@ next_yaml_read_file(struct yaml_read_handler_data *d)
 {
 #ifdef DO_WORDEXP
     struct yaml_read_file *f = d->f;
+    int rv;
 
  retry:
     f->filename = f->files.we_wordv[f->curr_file++];
-    f->f = fopen(f->filename, "r");
-    if (!f->f) {
+    rv = f_open(f->filename, DO_READ, 0, &f->ft);
+    if (rv) {
 	yaml_errout_d_f(d, f->prev_f,
-			"Unable to open file %s, skipping\n", f->filename);
+			"Unable to open file %s, skipping: %s", f->filename,
+			gensio_err_to_str(rv));
 	if (f->curr_file < f->files.we_wordc)
 	    goto retry;
 	return 1;
@@ -456,7 +512,7 @@ do_include(struct yconf *y, const char *ivalue)
 
     if (y->d->include_depth > 100) {
 	yaml_errout(y, "Too many nested includes, you probably have a"
-		    " circular include\n");
+		    " circular include");
 	return -1;
     }
 
@@ -473,24 +529,24 @@ do_include(struct yconf *y, const char *ivalue)
 
     case WRDE_BADCHAR:
 	yaml_errout(y, "Bad character in include directive");
-	return -1;
+	goto out_err;
 
     case WRDE_CMDSUB:
 	yaml_errout(y, "Command substitution not allowed in include directive");
-	return -1;
+	goto out_err;
 
     case WRDE_NOSPACE:
 	yaml_errout(y, "Out of memory processing include directive");
-	return -1;
+	goto out_err;
 
     case WRDE_SYNTAX:
 	yaml_errout(y, "Syntax error in include directive");
-	return -1;
+	goto out_err;
 
     case WRDE_BADVAL:
     default:
 	yaml_errout(y, "Unknown error in include directive");
-	return -1;
+	goto out_err;
     }
 
     f->closeme = true;
@@ -501,19 +557,24 @@ do_include(struct yconf *y, const char *ivalue)
     if (next_yaml_read_file(y->d))
 	cleanup_yaml_read_file(y->d);
     return 0;
+ out_err:
+    free(f);
+    return -1;
 #else
     yaml_errout(y, "Include is not supported on this system");
     return -1;
 #endif
 }
 
+#define ALLOC_INC 4096
 static struct yfile *
 lookup_filename_len(struct yconf *y, const char *filename, unsigned int len)
 {
     struct yfile *f = y->files;
-    int infd, rv;
-    char *name, *value = NULL;
-    struct stat stat;
+    ftype *ft = NULL;
+    int rv;
+    unsigned int buflen = 0, newlen, outlen, readleft;
+    char *name, *value = NULL, *nvalue = NULL;
 
     while (f && f->namelen == len && strncmp(f->name, filename, len) != 0)
 	f = f->next;
@@ -526,37 +587,53 @@ lookup_filename_len(struct yconf *y, const char *filename, unsigned int len)
 	return NULL;
     }
 
-    infd = open(name, O_RDONLY);
-    if (infd == -1) {
-	yaml_errout(y, "Error opening %s: %s", name, strerror(errno));
+    rv = f_open(name, DO_READ, 0, &ft);
+    if (rv) {
+	yaml_errout(y, "Error opening %s: %s", name, gensio_err_to_str(rv));
 	goto out_err;
     }
 
-    rv = fstat(infd, &stat);
-    if (rv == -1) {
-	yaml_errout(y, "Error stat-ing %s: %s", name, strerror(errno));
-	goto out_err;
-    }
+    do {
+	newlen = buflen + ALLOC_INC;
+	nvalue = realloc(value, newlen);
+	if (!nvalue) {
+	    yaml_errout(y, "Error allocating memory for file %s", name);
+	    rv = GE_NOMEM;
+	    goto out_err;
+	}
+	value = nvalue;
 
-    value = malloc(stat.st_size + 1);
-    if (!value) {
-	yaml_errout(y, "Error allocating memory for file %s", name);
-	goto out_err;
+	readleft = ALLOC_INC;
+	while (readleft > 0) {
+	    rv = f_read(ft, value + buflen, ALLOC_INC, &outlen);
+	    if (rv) {
+		if (rv == GE_REMCLOSE)
+		    break;
+		yaml_errout(y, "Error reading %s: %s", name,
+			    gensio_err_to_str(rv));
+		goto out_err;
+	    }
+	    buflen += outlen;
+	    readleft -= outlen;
+	}
+    } while (1);
+    if (buflen == newlen) {
+	nvalue = realloc(value, buflen + 1);
+	if (!nvalue) {
+	    yaml_errout(y, "Error allocating memory for file %s", name);
+	    rv = GE_NOMEM;
+	    goto out_err;
+	}
+	value = nvalue;
     }
-
-    rv = read(infd, value, stat.st_size);
-    if (rv == -1) {
-	yaml_errout(y, "Error reading %s: %s", name, strerror(errno));
-	goto out_err;
-    }
-    value[stat.st_size] = '\0';
+    value[buflen] = '\0';
 
     f = malloc(sizeof(*f));
     if (!f) {
 	yaml_errout(y, "Error allocating memory for file struct %s", name);
 	goto out_err;
     }
-    close(infd);
+    f_close(ft);
 
     f->name = name;
     f->namelen = strlen(name);
@@ -570,8 +647,8 @@ lookup_filename_len(struct yconf *y, const char *filename, unsigned int len)
     if (value)
 	free(value);
     free(name);
-    if (infd != -1)
-	close(infd);
+    if (ft)
+	f_close(ft);
     return NULL;
 }
 
@@ -617,35 +694,35 @@ add_option(struct yconf *y, const char *name, const char *option,
 }
 
 static int
-add_connection(struct yconf *y, const char *connection, const char *place)
+add_seq(struct yconf *y, const char *item, const char *place)
 {
-    if (y->curr_connection >= y->connections_len) {
-	unsigned int new_len = y->connections_len + 10;
-	char **new_connections = malloc(sizeof(char *) * new_len);
+    if (y->curr_seq >= y->seq_len) {
+	unsigned int new_len = y->seq_len + 10;
+	char **new_seq = malloc(sizeof(char *) * new_len);
 
-	if (!new_connections) {
+	if (!new_seq) {
 	    yaml_errout(y, "Out of memory allocating connection array for %s",
 			place);
 	    return -1;
 	}
-	memcpy(new_connections, y->connections,
-	       sizeof(char *) * y->connections_len);
-	free(y->connections);
-	y->connections = new_connections;
-	y->connections_len = new_len;
+	memcpy(new_seq, y->seq,
+	       sizeof(char *) * y->seq_len);
+	free(y->seq);
+	y->seq = new_seq;
+	y->seq_len = new_len;
     }
 
-    if (connection) {
-	y->connections[y->curr_connection] = strdup(connection);
-	if (!y->connections[y->curr_connection]) {
+    if (item) {
+	y->seq[y->curr_seq] = strdup(item);
+	if (!y->seq[y->curr_seq]) {
 	    yaml_errout(y, "Out of memory allocating connection %s for %s",
-			connection, place);
+			item, place);
 	    return -1;
 	}
     } else {
-	y->connections[y->curr_connection] = NULL;
+	y->seq[y->curr_seq] = NULL;
     }
-    y->curr_connection++;
+    y->curr_seq++;
     return 0;
 }
 
@@ -673,6 +750,14 @@ DECL_KEYVAL(class);
 #define KEYVAL_OFFSET(y)			\
     ((char **) (((char *) (y)) + (y)->keyval_info->keyval_offset))
 
+static struct scalar_next_state sc_defaults[] = {
+    { "value", IN_MAIN_MAP_KEYVAL, WHICH_INFO_KEYVAL,
+      .keyval_info = &keyval_value },
+    { "class", IN_MAIN_MAP_KEYVAL, WHICH_INFO_KEYVAL,
+      .keyval_info = &keyval_class },
+    {}
+};
+
 static struct scalar_next_state sc_default[] = {
     { "name", IN_MAIN_MAP_KEYVAL, WHICH_INFO_KEYVAL,
       .keyval_info = &keyval_name },
@@ -735,15 +820,26 @@ static struct scalar_next_state sc_led[] = {
     {}
 };
 
+static struct option_info admin_option_info = {
+    "admin",
+    IN_MAIN_MAP,
+};
+
 static struct scalar_next_state sc_admin[] = {
     { "accepter", IN_MAIN_MAP_KEYVAL, WHICH_INFO_KEYVAL,
       .keyval_info = &keyval_accepter },
     { "options", IN_OPTIONS, WHICH_INFO_OPTION,
-      .option_info = &led_option_info },
+      .option_info = &admin_option_info },
     {}
 };
 
 enum main_map_types {
+    MAIN_MAP_DEFAULTS,
+    MAIN_MAP_ADMINS,
+    MAIN_MAP_LEDS,
+    MAIN_MAP_CONNECTIONS,
+    MAIN_MAP_ROTATORS,
+
     MAIN_MAP_DEFAULT,
     MAIN_MAP_DELDEFAULT,
     MAIN_MAP_CONNECTION,
@@ -752,41 +848,74 @@ enum main_map_types {
     MAIN_MAP_ADMIN
 };
 
+static struct map_info sc_defaults_map = {
+    "defaults", sc_defaults, MAIN_MAP_DEFAULTS, false, false
+};
+
+static struct map_info sc_admins_map = {
+    "admins", sc_admin, MAIN_MAP_ADMINS, false, false
+};
+
+static struct map_info sc_leds_map = {
+    "leds", sc_led, MAIN_MAP_LEDS, false, false
+};
+
+static struct map_info sc_connections_map = {
+    "connections", sc_connection, MAIN_MAP_CONNECTIONS, false, false
+};
+
+static struct map_info sc_rotators_map = {
+    "rotators", sc_rotator, MAIN_MAP_ROTATORS, false, false
+};
+
 static struct map_info sc_default_map = {
-    "default", sc_default, MAIN_LEVEL, MAIN_MAP_DEFAULT, false
+    "default", sc_default, MAIN_MAP_DEFAULT, false, false
 };
 
 static struct map_info sc_deldefault_map = {
-    "deldefault", sc_deldefault, MAIN_LEVEL, MAIN_MAP_DELDEFAULT, false
+    "deldefault", sc_deldefault, MAIN_MAP_DELDEFAULT, false, false
 };
 
 static struct map_info sc_connection_map = {
-    "connection", sc_connection, MAIN_LEVEL, MAIN_MAP_CONNECTION, true
+    "connection", sc_connection, MAIN_MAP_CONNECTION, true, false
 };
 
 static struct map_info sc_rotator_map = {
-    "rotator", sc_rotator, MAIN_LEVEL, MAIN_MAP_ROTATOR, true
+    "rotator", sc_rotator, MAIN_MAP_ROTATOR, true, false
 };
 
 static struct map_info sc_led_map = {
-    "led", sc_led, MAIN_LEVEL, MAIN_MAP_LED, true
+    "led", sc_led, MAIN_MAP_LED, true, false
 };
 
 static struct map_info sc_admin_map = {
-    "admin", sc_admin, MAIN_LEVEL, MAIN_MAP_ADMIN, false
+    "admin", sc_admin, MAIN_MAP_ADMIN, false, true
 };
 
 static struct scalar_next_state sc_main[] = {
-    { "define", IN_DEFINE },
+    { "defines", IN_DEFINES, .version = 2 },
+    { "defaults", IN_DEFAULTS, .version = 2 },
+    { "delete_defaults", IN_DELDEFAULTS, .version = 2 },
+    { "admins", IN_ADMINS, .version = 2 },
+    { "leds", IN_LEDS, .version = 2 },
+    { "connections", IN_CONNECTIONS, .version = 2 },
+    { "rotators", IN_ROTATORS, .version = 2 },
+
     { "include", IN_INCLUDE },
-    { "default", IN_MAIN_NAME, WHICH_INFO_MAP, .map_info = &sc_default_map },
+
+    { "define", IN_DEFINE, .version = 1 },
+    { "default", IN_MAIN_NAME, WHICH_INFO_MAP, .map_info = &sc_default_map,
+      .version = 1 },
     { "delete_default", IN_MAIN_NAME, WHICH_INFO_MAP,
-      .map_info = &sc_deldefault_map },
+      .map_info = &sc_deldefault_map, .version = 1 },
     { "connection", IN_MAIN_NAME, WHICH_INFO_MAP,
-      .map_info = &sc_connection_map },
-    { "rotator", IN_MAIN_NAME, WHICH_INFO_MAP, .map_info = &sc_rotator_map },
-    { "led", IN_MAIN_NAME, WHICH_INFO_MAP, .map_info = &sc_led_map },
-    { "admin", IN_MAIN_NAME, WHICH_INFO_MAP, .map_info = &sc_admin_map },
+      .map_info = &sc_connection_map, .version = 1 },
+    { "rotator", IN_MAIN_NAME, WHICH_INFO_MAP, .map_info = &sc_rotator_map,
+      .version = 1},
+    { "led", IN_MAIN_NAME, WHICH_INFO_MAP, .map_info = &sc_led_map,
+      .version = 1 },
+    { "admin", IN_MAIN_NAME, WHICH_INFO_MAP, .map_info = &sc_admin_map,
+      .version = 1 },
     {}
 };
 
@@ -816,6 +945,13 @@ scalar_next_state(struct yconf *y,
 {
     while (s->name) {
 	if (strcasecmp(s->name, scalar) == 0) {
+	    if (y->version && s->version && y->version != s->version) {
+		yaml_errout(y, "Mixed version in yaml file");
+		y->state = PARSE_ERR;
+		return;
+	    }
+	    if (s->version)
+		y->version = s->version;
 	    switch (s->infotype) {
 	    case WHICH_INFO_NONE: break;
 	    case WHICH_INFO_OPTION: y->option_info = s->option_info; break;
@@ -841,11 +977,17 @@ process_scalar(struct yconf *y, const char *iscalar)
     char *rv = NULL, *out = NULL;
     unsigned int len = 0, alen;
     int state = 0;
+    char actchar;
+
+    if (y->version == 1)
+	actchar = '*';
+    else
+	actchar = '=';
 
  restart:
     for (s = iscalar; *s; s++) {
 	if (state == 0) {
-	    if (*s == '*')
+	    if (*s == actchar)
 		state = 1;
 	    else {
 		if (out)
@@ -853,20 +995,22 @@ process_scalar(struct yconf *y, const char *iscalar)
 		len++;
 	    }
 	} else if (state == 1) {
-	    /* Last character was a '*' */
+	    /* Last character was a actchar */
 	    if (*s == '(') {
 		start = s + 1;
 		state = 2;
 	    } else if (*s == '{') {
 		start = s + 1;
 		state = 3;
-	    } else if (*s == '*') {
+	    } else if (*s == actchar) {
+		/* Two actchars map to a single actchar */
 		if (out)
 		    *out++ = *s;
-		len++; /* Stay in state 1 */
+		len++;
+		state = 0;
 	    } else {
 		if (out) {
-		    *out++ = '*';
+		    *out++ = actchar;
 		    *out++ = *s;
 		}
 		len += 2;
@@ -874,10 +1018,10 @@ process_scalar(struct yconf *y, const char *iscalar)
 	    }
 	} else if (state == 2) {
 	    /* We are in a '*(' */
-	    if (start == s && *s == '*') {
+	    if (start == s && *s == actchar) {
 		/* '*(*' outputs a '*(' */
 		if (out) {
-		    *out++ = '*';
+		    *out++ = actchar;
 		    *out++ = '(';
 		}
 		len += 2;
@@ -901,10 +1045,10 @@ process_scalar(struct yconf *y, const char *iscalar)
 	    }
 	} else if (state == 3) {
 	    /* We are in a '*{' */
-	    if (start == s && *s == '*') {
+	    if (start == s && *s == actchar) {
 		/* '*{*' outputs a '*{' */
 		if (out) {
-		    *out++ = '*';
+		    *out++ = actchar;
 		    *out++ = '{';
 		}
 		len += 2;
@@ -960,15 +1104,95 @@ yhandle_scalar(struct yconf *y, const char *anchor, const char *iscalar)
     case MAIN_LEVEL:
 	scalar_next_state(y, sc_main, scalar);
 	if (y->state == PARSE_ERR) {
-	    yaml_errout(y, "Invalid token at the main level: %s\n", scalar);
+	    yaml_errout(y, "Invalid token at the main level: %s", scalar);
 	    goto out_err;
 	}
 	break;
 
+    case IN_DEFINES_MAP:
+	y->name = strdup(scalar);
+	if (!y->name) {
+	    yaml_errout(y, "Out of memory allocating name");
+	    goto out_err;
+	}
+	y->state = IN_DEFINES_VAL;
+	break;
+
+    case IN_DEFINES_VAL:
+	anchor_allowed = true;
+	if (add_alias(y, y->name, scalar))
+	    goto out_err;
+	if (anchor) {
+	    if (add_alias(y, anchor, scalar))
+		goto out_err;
+	}
+	dofree(&y->name);
+	y->state = IN_DEFINES_MAP;
+	break;
+
+    case IN_DEFAULTS_MAP:
+	y->name = strdup(scalar);
+	if (!y->name) {
+	    yaml_errout(y, "Out of memory allocating name");
+	    goto out_err;
+	}
+	y->state = IN_DEFAULTS_INST;
+	break;
+
+    case IN_DELDEFAULTS_MAP:
+	y->name = strdup(scalar);
+	if (!y->name) {
+	    yaml_errout(y, "Out of memory allocating name");
+	    goto out_err;
+	}
+	y->state = IN_DELDEFAULTS_SEQ;
+	break;
+
+    case IN_DELDEFAULTS_VAL:
+	if (add_seq(y, scalar, "delete_defaults"))
+	    goto out_err;
+	break;
+
+    case IN_ADMINS_MAP:
+	y->name = strdup(scalar);
+	if (!y->name) {
+	    yaml_errout(y, "Out of memory allocating name");
+	    goto out_err;
+	}
+	y->state = IN_ADMINS_INST;
+	break;
+
+    case IN_LEDS_MAP:
+	y->name = strdup(scalar);
+	if (!y->name) {
+	    yaml_errout(y, "Out of memory allocating name");
+	    goto out_err;
+	}
+	y->state = IN_LEDS_INST;
+	break;
+
+    case IN_CONNECTIONS_MAP:
+	y->name = strdup(scalar);
+	if (!y->name) {
+	    yaml_errout(y, "Out of memory allocating name");
+	    goto out_err;
+	}
+	y->state = IN_CONNECTIONS_INST;
+	break;
+
+    case IN_ROTATORS_MAP:
+	y->name = strdup(scalar);
+	if (!y->name) {
+	    yaml_errout(y, "Out of memory allocating name");
+	    goto out_err;
+	}
+	y->state = IN_ROTATORS_INST;
+	break;
+
     case IN_DEFINE:
 	anchor_allowed = true;
 	if (!anchor)
-	    yaml_errout(y, "No anchor for define, define ignored\n");
+	    yaml_errout(y, "No anchor for define, define ignored");
 	else {
 	    if (add_alias(y, anchor, scalar))
 		goto out_err;
@@ -1034,7 +1258,7 @@ yhandle_scalar(struct yconf *y, const char *anchor, const char *iscalar)
 	break;
 
     case IN_ROTATOR_CONNECTIONS_SEQ:
-	if (add_connection(y, scalar, "rotator"))
+	if (add_seq(y, scalar, "rotator"))
 	    goto out_err;
 	break;
 
@@ -1044,7 +1268,7 @@ yhandle_scalar(struct yconf *y, const char *anchor, const char *iscalar)
     }
 
     if (anchor && !anchor_allowed)
-	yaml_errout(y, "Anchor on non-scalar ignored\n");
+	yaml_errout(y, "Anchor on non-scalar ignored");
 
     free(scalar);
     return 0;
@@ -1062,6 +1286,10 @@ yhandle_seq_start(struct yconf *y)
 	y->state = IN_ROTATOR_CONNECTIONS_SEQ;
 	break;
 
+    case IN_DELDEFAULTS_SEQ:
+	y->state = IN_DELDEFAULTS_VAL;
+	break;
+
     default:
 	yaml_errout(y, "Unexpected sequence start: %d", y->state);
 	return -1;
@@ -1073,17 +1301,36 @@ yhandle_seq_start(struct yconf *y)
 static int
 yhandle_seq_end(struct yconf *y)
 {
+    unsigned int i;
+    int err, reterr = 0;
+
     switch (y->state) {
     case IN_ROTATOR_CONNECTIONS_SEQ:
 	y->state = IN_MAIN_MAP;
 	break;
 
+    case IN_DELDEFAULTS_VAL:
+	if (y->curr_seq == 0) {
+	    yaml_errout(y, "No class(es) given in delete_defaults");
+	    return -1;
+	}
+	for (i = 0; i < y->curr_seq; i++) {
+	    err = gensio_del_default(so, y->seq[i], y->name, false);
+	    if (err) {
+		reterr = -1;
+		yaml_errout(y, "Unable to delete default name %s:%s: %s",
+			    y->seq[i], y->name, gensio_err_to_str(err));
+	    }
+	}
+	y->state = IN_DELDEFAULTS_MAP;
+	break;
+
     default:
 	yaml_errout(y, "Unexpected sequence end: %d", y->state);
 	return -1;
     }
 
-    return 0;
+    return reterr;
 }
 
 static int
@@ -1091,24 +1338,80 @@ yhandle_mapping_start(struct yconf *y)
 {
     switch (y->state) {
     case BEGIN_DOC:
+    case END_DOC: /* Allow multiple documents, just go on. */
 	y->state = MAIN_LEVEL;
 	break;
 
+    case IN_DEFINES:
+	y->state = IN_DEFINES_MAP;
+	break;
+
+    case IN_DEFAULTS:
+	y->state = IN_DEFAULTS_MAP;
+	break;
+
+    case IN_DEFAULTS_INST:
+	y->map_info = &sc_defaults_map;
+	y->state = IN_MAIN_MAP;
+	break;
+
+    case IN_DELDEFAULTS:
+	y->state = IN_DELDEFAULTS_MAP;
+	break;
+
+    case IN_ADMINS:
+	y->state = IN_ADMINS_MAP;
+	break;
+
+    case IN_ADMINS_INST:
+	y->map_info = &sc_admins_map;
+	y->state = IN_MAIN_MAP;
+	break;
+
+    case IN_LEDS:
+	y->state = IN_LEDS_MAP;
+	break;
+
+    case IN_LEDS_INST:
+	y->map_info = &sc_leds_map;
+	y->state = IN_MAIN_MAP;
+	break;
+
+    case IN_CONNECTIONS:
+	y->state = IN_CONNECTIONS_MAP;
+	break;
+
+    case IN_CONNECTIONS_INST:
+	y->map_info = &sc_connections_map;
+	y->state = IN_MAIN_MAP;
+	break;
+
+    case IN_ROTATORS:
+	y->state = IN_ROTATORS_MAP;
+	break;
+
+    case IN_ROTATORS_INST:
+	y->map_info = &sc_rotators_map;
+	y->state = IN_MAIN_MAP;
+	break;
+
     case IN_MAIN_NAME:
-	if (y->map_info->needs_anchor) {
+	if (y->map_info->needs_anchor || y->map_info->wants_anchor) {
 	    char *anchor = (char *) y->d->f->e.data.mapping_start.anchor;
 
-	    if (!anchor) {
+	    if (!anchor && y->map_info->needs_anchor) {
 		yaml_errout(y, "Main mapping requires an anchor for the name");
 		return -1;
 	    }
-	    y->name = strdup(anchor);
-	    if (!y->name) {
-		yaml_errout(y, "Out of memory allocating name");
-		return -1;
+	    if (anchor) {
+		y->name = strdup(anchor);
+		if (!y->name) {
+		    yaml_errout(y, "Out of memory allocating name");
+		    return -1;
+		}
+		if (add_alias(y, anchor, anchor))
+		    return -1;
 	    }
-	    if (add_alias(y, anchor, anchor))
-		return -1;
 	}
 	y->state = IN_MAIN_MAP;
 	break;
@@ -1136,13 +1439,121 @@ yhandle_mapping_end(struct yconf *y)
 	y->state = END_DOC;
 	break;
 
+    case IN_DEFINES_MAP:
+    case IN_DEFAULTS_MAP:
+    case IN_DELDEFAULTS_MAP:
+    case IN_LEDS_MAP:
+    case IN_ADMINS_MAP:
+    case IN_CONNECTIONS_MAP:
+    case IN_ROTATORS_MAP:
+	y->state = MAIN_LEVEL;
+	break;
+
     case IN_MAIN_MAP:
 	switch (y->map_info->map_type) {
+	case MAIN_MAP_DEFAULTS:
+	    if (!y->value) {
+		yaml_errout(y, "No value given in default");
+		return -1;
+	    }
+	    err = gensio_set_default(so, y->class, y->name, y->value, 0);
+	    if (err) {
+		yaml_errout(y, "Unable to set default name %s:%s:%s: %s",
+			    y->class ? y->class : "",
+			    y->name, y->value, gensio_err_to_str(err));
+		return -1;
+	    }
+	    y->state = IN_DEFAULTS_MAP;
+	    yconf_cleanup_main(y);
+	    break;
+
+	case MAIN_MAP_ADMINS:
+	    if (!y->accepter) {
+		yaml_errout(y, "No accepter given in admin");
+		return -1;
+	    }
+	    /* NULL terminate the options. */
+	    if (add_option(y, NULL, NULL, "admin"))
+		return -1;
+	    controller_init(y->accepter, y->name, (const char **) y->options,
+			    &y->sub_errout);
+	    y->state = IN_ADMINS_MAP;
+	    yconf_cleanup_main(y);
+	    break;
+
+	case MAIN_MAP_LEDS:
+	    if (!y->driver) {
+		yaml_errout(y, "No driver given in led");
+		return -1;
+	    }
+	    /* NULL terminate the options. */
+	    if (add_option(y, NULL, NULL, "led"))
+		return -1;
+	    err = add_led(y->name, y->driver,
+			  (const char **) y->options,
+			  y->d->f->e.start_mark.line, &y->sub_errout);
+	    y->state = IN_LEDS_MAP;
+	    yconf_cleanup_main(y);
+	    break;
+
+	case MAIN_MAP_CONNECTIONS:
+	    if (!y->accepter) {
+		yaml_errout(y, "No accepter given in connection");
+		return -1;
+	    }
+	    if (!y->connector) {
+		yaml_errout(y, "No connector given in connection");
+		return -1;
+	    }
+	    /* NULL terminate the options. */
+	    if (add_option(y, NULL, NULL, "connection"))
+		return -1;
+	    portconfig(&y->sub_errout, y->name, y->accepter,
+		       y->enable ? "raw" : "off", y->timeout,
+		       y->connector, (const char **) y->options);
+	    y->state = IN_CONNECTIONS_MAP;
+	    yconf_cleanup_main(y);
+	    break;
+	
+	case MAIN_MAP_ROTATORS:
+	    if (!y->accepter) {
+		yaml_errout(y, "No accepter given in rotator");
+		return -1;
+	    }
+	    if (y->curr_seq == 0) {
+		yaml_errout(y, "No connections given in rotator");
+		return -1;
+	    }
+	    /* NULL terminate the connections. */
+	    if (add_seq(y, NULL, "rotator"))
+		return -1;
+	    err = gensio_argv_copy(so, (const char **) y->seq,
+				   &argc, &argv);
+	    if (err) {
+		yaml_errout(y, "Unable to allocate rotator connections");
+		return -1;
+	    }
+	    /* NULL terminate the options. */
+	    if (add_option(y, NULL, NULL, "rotator"))
+		return -1;
+	    err = add_rotator(&y->sub_errout, y->name, y->accepter, argc, argv,
+			      (const char **) y->options,
+			      y->d->f->e.start_mark.line);
+	    if (err)
+		gensio_argv_free(so, argv);
+	    y->state = IN_ROTATORS_MAP;
+	    yconf_cleanup_main(y);
+	    break;
+
 	case MAIN_MAP_DEFAULT:
 	    if (!y->name) {
 		yaml_errout(y, "No name given in default");
 		return -1;
 	    }
+	    if (!y->value) {
+		yaml_errout(y, "No value given in default");
+		return -1;
+	    }
 	    err = gensio_set_default(so, y->class, y->name, y->value, 0);
 	    if (err) {
 		yaml_errout(y, "Unable to set default name %s:%s:%s: %s",
@@ -1165,9 +1576,8 @@ yhandle_mapping_end(struct yconf *y)
 	    }
 	    err = gensio_del_default(so, y->class, y->name, false);
 	    if (err) {
-		yaml_errout(y, "Unable to set default name %s:%s:%s: %s",
-			    y->class ? y->class : "",
-			    y->name, y->value, gensio_err_to_str(err));
+		yaml_errout(y, "Unable to delete default name %s:%s: %s",
+			    y->class, y->name, gensio_err_to_str(err));
 		return -1;
 	    }
 	    y->state = MAIN_LEVEL;
@@ -1206,14 +1616,14 @@ yhandle_mapping_end(struct yconf *y)
 		yaml_errout(y, "No accepter given in rotator");
 		return -1;
 	    }
-	    if (y->curr_connection == 0) {
+	    if (y->curr_seq == 0) {
 		yaml_errout(y, "No connections given in rotator");
 		return -1;
 	    }
 	    /* NULL terminate the connections. */
-	    if (add_connection(y, NULL, "rotator"))
+	    if (add_seq(y, NULL, "rotator"))
 		return -1;
-	    err = gensio_argv_copy(so, (const char **) y->connections,
+	    err = gensio_argv_copy(so, (const char **) y->seq,
 				   &argc, &argv);
 	    if (err) {
 		yaml_errout(y, "Unable to allocate rotator connections");
@@ -1245,7 +1655,7 @@ yhandle_mapping_end(struct yconf *y)
 		return -1;
 	    err = add_led(y->name, y->driver,
 			  (const char **) y->options,
-			  y->d->f->e.start_mark.line);
+			  y->d->f->e.start_mark.line, &y->sub_errout);
 	    y->state = MAIN_LEVEL;
 	    yconf_cleanup_main(y);
 	    break;
@@ -1258,7 +1668,7 @@ yhandle_mapping_end(struct yconf *y)
 	    /* NULL terminate the options. */
 	    if (add_option(y, NULL, NULL, "admin"))
 		return -1;
-	    controller_init(y->accepter, (const char **) y->options,
+	    controller_init(y->accepter, y->name, (const char **) y->options,
 			    &y->sub_errout);
 	    y->state = MAIN_LEVEL;
 	    yconf_cleanup_main(y);
@@ -1302,7 +1712,7 @@ process_buffer(struct yaml_read_handler_data *d,
 	case '\'':
 	case '"':
 	    if (d->in_quote == '"' && d->in_escape)
-		/* \" in "" doens't end the quotes. */
+		/* \" in "" doesn't end the quotes. */
 		goto normal_char;
 	    if (d->in_quote == c)
 		d->in_quote = 0;
@@ -1341,26 +1751,31 @@ yaml_read_handler(void *data, unsigned char *buffer, size_t size,
 		  size_t *size_read)
 {
     struct yaml_read_handler_data *d = data;
+    unsigned int readlen;
+    int rv;
 
     *size_read = 0;
 
-    if (d->f->f) {
-	*size_read = fread(buffer, 1, size, d->f->f);
-	if (*size_read < size) {
-	    if (ferror(d->f->f)) {
+    if (d->f->ft) {
+	do {
+	    rv = f_read(d->f->ft, buffer, size, &readlen);
+	    if (rv == GE_REMCLOSE) {
+		/*
+		 * End of file handling in the main routine will move to
+		 * the next file as necessary.
+		 */
+		break;
+	    }
+	    if (rv) {
 		yaml_errout(d->y, "Error reading input file: %s",
-			    strerror(errno));
+			    gensio_err_to_str(rv));
 		return 0;
 	    }
-	    /* End of file */
-	    size -= *size_read;
-	    buffer += *size_read;
+	    *size_read += readlen;
+	    size -= readlen;
+	    buffer += readlen;
+	} while(size > 0);
 
-	    /*
-	     * End of file handling in the main routine will move to
-	     * the next file as necessary.
-	     */
-	}
 	return 1;
     }
 
@@ -1400,7 +1815,7 @@ yaml_read_handler(void *data, unsigned char *buffer, size_t size,
 }
 
 int
-yaml_readconfig(FILE *file, char *filename,
+yaml_readconfig(ftype *file, char *filename,
 		char **config_lines, unsigned int num_config_lines,
 		struct absout *errout)
 {
@@ -1415,18 +1830,19 @@ yaml_readconfig(FILE *file, char *filename,
     y.options = malloc(sizeof(char *) * 10);
     if (!y.options) {
 	errout->out(errout, "Out of memory allocating options array");
-	return ENOMEM;
+	return GE_NOMEM;
     }
     y.options_len = 10;
-    y.connections = malloc(sizeof(char *) * 10);
-    if (!y.connections) {
+    y.seq = malloc(sizeof(char *) * 10);
+    if (!y.seq) {
 	free(y.options);
 	errout->out(errout, "Out of memory allocating connection array");
-	return ENOMEM;
+	return GE_NOMEM;
     }
-    y.connections_len = 10;
+    y.seq_len = 10;
     y.state = BEGIN_DOC;
     y.sub_errout.out = sub_errout;
+    y.sub_errout.vout = sub_verrout;
     y.sub_errout.data = &y;
     y.d = &d;
 
@@ -1439,29 +1855,50 @@ yaml_readconfig(FILE *file, char *filename,
 
     memset(&f, 0, sizeof(f));
     f.filename = filename;
-    f.f = file;
+    f.ft = file;
     yaml_parser_initialize(&f.parser);
     yaml_parser_set_input(&f.parser, yaml_read_handler, &d);
 
     while (!done && !err) {
 	if (!yaml_parser_parse(&y.d->f->parser, &y.d->f->e)) {
 	    yaml_errout(&y, y.d->f->parser.problem);
-	    err = EINVAL;
+	    err = GE_INVAL;
 	    break;
 	}
 
 	switch (y.d->f->e.type) {
 	case YAML_NO_EVENT:
+#if DEBUG
+	    printf("YAML_NO_EVENT\n");
+#endif
+	    break;
+
 	case YAML_STREAM_START_EVENT:
+#if DEBUG
+	    printf("YAML_STREAM_START_EVENT\n");
+#endif
+	    break;
+
 	case YAML_DOCUMENT_START_EVENT:
+#if DEBUG
+	    printf("YAML_DOCUMENT_START_EVENT\n");
+#endif
+	    break;
+
 	case YAML_DOCUMENT_END_EVENT:
+#if DEBUG
+	    printf("YAML_DOCUMENT_END_EVENT\n");
+#endif
 	    break;
 
 	case YAML_STREAM_END_EVENT:
+#if DEBUG
+	    printf("YAML_STREAM_END_EVENT\n");
+#endif
 	    if (y.state != END_DOC) {
 		yaml_errout(&y, "yaml file ended in invalid state: %d",
 			    y.state);
-		err = EINVAL;
+		err = GE_INVAL;
 	    }
 	    done = true;
 	    break;
@@ -1470,16 +1907,19 @@ yaml_readconfig(FILE *file, char *filename,
 	    struct alias *a;
 #if DEBUG
 	    printf("YAML_ALIAS_EVENT\n");
-	    printf(" anc: '%s'\n", y.e.data.alias.anchor);
+	    printf(" anc: '%s'\n", y.d->f->e.data.alias.anchor);
 #endif
 	    a = lookup_alias(&y, (char *) y.d->f->e.data.alias.anchor);
 	    if (!a) {
 		yaml_errout(&y, "Unable to find alias '%s'",
 			    y.d->f->e.data.alias.anchor);
-		err = EINVAL;
+		err = GE_INVAL;
 	    } else {
+#if DEBUG
+		printf(" val: '%s'\n", a->value);
+#endif
 		if (yhandle_scalar(&y, NULL, a->value))
-		    err = EINVAL;
+		    err = GE_INVAL;
 	    }
 	    break;
 	}
@@ -1487,23 +1927,23 @@ yaml_readconfig(FILE *file, char *filename,
 	case YAML_SCALAR_EVENT:
 #if DEBUG
 	    printf("YAML_SCALAR_EVENT\n");
-	    printf(" anc: '%s'\n", y.e.data.scalar.anchor);
-	    printf(" tag: '%s'\n", y.e.data.scalar.tag);
-	    printf(" val: '%s'\n", y.e.data.scalar.value);
+	    printf(" anc: '%s'\n", y.d->f->e.data.scalar.anchor);
+	    printf(" tag: '%s'\n", y.d->f->e.data.scalar.tag);
+	    printf(" val: '%s'\n", y.d->f->e.data.scalar.value);
 #endif
 	    if (yhandle_scalar(&y, (char *) y.d->f->e.data.scalar.anchor,
 			       (char *) y.d->f->e.data.scalar.value))
-		err = EINVAL;
+		err = GE_INVAL;
 	    break;
 
 	case YAML_SEQUENCE_START_EVENT:
 #if DEBUG
 	    printf("YAML_SEQUENCE_START_EVENT\n");
-	    printf(" anc: '%s'\n", y.e.data.sequence_start.anchor);
-	    printf(" tag: '%s'\n", y.e.data.sequence_start.tag);
+	    printf(" anc: '%s'\n", y.d->f->e.data.sequence_start.anchor);
+	    printf(" tag: '%s'\n", y.d->f->e.data.sequence_start.tag);
 #endif
 	    if (yhandle_seq_start(&y))
-		err = EINVAL;
+		err = GE_INVAL;
 	    break;
 
 	case YAML_SEQUENCE_END_EVENT:
@@ -1511,17 +1951,17 @@ yaml_readconfig(FILE *file, char *filename,
 	    printf("YAML_SEQUENCE_END_EVENT\n");
 #endif
 	    if (yhandle_seq_end(&y))
-		err = EINVAL;
+		err = GE_INVAL;
 	    break;
 
 	case YAML_MAPPING_START_EVENT:
 #if DEBUG
 	    printf("YAML_MAPPING_START_EVENT\n");
-	    printf(" anc: '%s'\n", y.e.data.mapping_start.anchor);
-	    printf(" tag: '%s'\n", y.e.data.mapping_start.tag);
+	    printf(" anc: '%s'\n", y.d->f->e.data.mapping_start.anchor);
+	    printf(" tag: '%s'\n", y.d->f->e.data.mapping_start.tag);
 #endif
 	    if (yhandle_mapping_start(&y))
-		err = EINVAL;
+		err = GE_INVAL;
 	    break;
 
 	case YAML_MAPPING_END_EVENT:
@@ -1529,7 +1969,7 @@ yaml_readconfig(FILE *file, char *filename,
 	    printf("YAML_MAPPING_END_EVENT\n");
 #endif
 	    if (yhandle_mapping_end(&y))
-		err = EINVAL;
+		err = GE_INVAL;
 	    break;
 	}
 
@@ -1543,7 +1983,7 @@ yaml_readconfig(FILE *file, char *filename,
 	    if (d.f) {
 		done = false;
 		yaml_parser_delete(&d.f->parser);
-		fclose(d.f->f);
+		f_close(d.f->ft);
 		if (next_yaml_read_file(&d))
 		    goto continue_clean;
 	    }
@@ -1555,7 +1995,8 @@ yaml_readconfig(FILE *file, char *filename,
 
     yconf_cleanup_main(&y);
     free(y.options);
-    free(y.connections);
+    free(y.seq);
+    y.seq_len = 0;
     while (y.aliases) {
 	struct alias *a = y.aliases;
 	y.aliases = a->next;
